@@ -1,38 +1,55 @@
 import logging
 import os
-import pykube
+import platform
 import random
-import requests
 import socket
 import subprocess
-import sys
 import time
 from contextlib import contextmanager
-from typing import Generator, Optional, Union
-
 from pathlib import Path
+from typing import Generator
+from typing import Optional
+from typing import Union
+
+import pykube
+import requests
 
 
-KIND_VERSION = "v0.9.0"
-KUBECTL_VERSION = "v1.19.1"
+KIND_VERSION = os.environ.get("KIND_VERSION", "v0.14.0")
+KUBECTL_VERSION = os.environ.get("KUBECTL_VERSION", "v1.24.2")
 
 
 class KindCluster:
-    def __init__(self, name: str, kubeconfig: Optional[Path] = None):
+    def __init__(
+        self,
+        name: str,
+        kubeconfig: Optional[Path] = None,
+        image: Optional[str] = None,
+        kind_path: Optional[Path] = None,
+        kubectl_path: Optional[Path] = None,
+    ):
         self.name = name
+        self.image = image
         path = Path(".pytest-kind")
         self.path = path / name
         self.path.mkdir(parents=True, exist_ok=True)
-        self.kind_path = self.path / "kind"
-        self.kubectl_path = self.path / "kubectl"
         self.kubeconfig_path = kubeconfig or (self.path / "kubeconfig")
+        self.kind_path = kind_path or (self.path / f"kind-{KIND_VERSION}")
+        self.platform = platform.system().lower()
+        if self.platform == "windows":
+            self.kubectl_path = kubectl_path or (
+                self.path / f"kubectl-{KUBECTL_VERSION}.exe"
+            )
+        else:
+            self.kubectl_path = kubectl_path or (
+                self.path / f"kubectl-{KUBECTL_VERSION}"
+            )
 
     def ensure_kind(self):
         if not self.kind_path.exists():
-            osname = sys.platform  # "linux" or "darwin"
             url = os.getenv(
                 "KIND_DOWNLOAD_URL",
-                f"https://github.com/kubernetes-sigs/kind/releases/download/{KIND_VERSION}/kind-{osname}-amd64",
+                f"https://github.com/kubernetes-sigs/kind/releases/download/{KIND_VERSION}/kind-{self.platform}-amd64",
             )
             logging.info(f"Downloading {url}..")
             tmp_file = self.kind_path.with_suffix(".tmp")
@@ -47,11 +64,16 @@ class KindCluster:
 
     def ensure_kubectl(self):
         if not self.kubectl_path.exists():
-            osname = sys.platform  # "linux" or "darwin"
-            url = os.getenv(
-                "KUBECTL_DOWNLOAD_URL",
-                f"https://storage.googleapis.com/kubernetes-release/release/{KUBECTL_VERSION}/bin/{osname}/amd64/kubectl",
-            )
+            if self.platform == "windows":
+                url = os.getenv(
+                    "KUBECTL_DOWNLOAD_URL",
+                    f"https://dl.k8s.io/release/{KUBECTL_VERSION}/bin/{self.platform}/amd64/kubectl.exe",
+                )
+            else:
+                url = os.getenv(
+                    "KUBECTL_DOWNLOAD_URL",
+                    f"https://dl.k8s.io/release/{KUBECTL_VERSION}/bin/{self.platform}/amd64/kubectl",
+                )
             logging.info(f"Downloading {url}..")
             tmp_file = self.kubectl_path.with_suffix(".tmp")
             with requests.get(url, stream=True) as r:
@@ -64,7 +86,7 @@ class KindCluster:
             tmp_file.rename(self.kubectl_path)
 
     def create(self, config_file: Union[str, Path] = None):
-        """Create the kind cluster if it does not exist (otherwise re-use)"""
+        """Create the kind cluster if it does not exist (otherwise re-use)."""
         self.ensure_kind()
 
         self.kubeconfig_path.touch(0o600, exist_ok=True)
@@ -88,6 +110,11 @@ class KindCluster:
                     f"--kubeconfig={self.kubeconfig_path}",
                 ]
 
+                if self.image:
+                    create_cmd += [
+                        f"--image={self.image}",
+                    ]
+
                 if config_file:
                     create_cmd += ["--config", str(config_file)]
 
@@ -117,11 +144,11 @@ class KindCluster:
         )
 
     def kubectl(self, *args: str, **kwargs) -> str:
-        """Run a kubectl command against the cluster and return the output as string"""
+        """Run a kubectl command against the cluster and return the output as string."""
         self.ensure_kubectl()
         return subprocess.check_output(
             [str(self.kubectl_path), *args],
-            env={"KUBECONFIG": str(self.kubeconfig_path)},
+            env={**os.environ, "KUBECONFIG": str(self.kubeconfig_path)},
             encoding="utf-8",
             **kwargs,
         )
@@ -135,7 +162,7 @@ class KindCluster:
         local_port: int = None,
         retries: int = 10,
     ) -> Generator[int, None, None]:
-        """Run "kubectl port-forward" for the given service/pod and use a random local port"""
+        """Run "kubectl port-forward" for the given service/pod and use a random local port."""
         port_to_use: int
         proc = None
         for i in range(retries):
@@ -178,7 +205,7 @@ class KindCluster:
                 proc.kill()
 
     def delete(self):
-        """Delete the kind cluster ("kind delete cluster")"""
+        """Delete the kind cluster ("kind delete cluster")."""
         logging.info(f"Deleting cluster {self.name}..")
         subprocess.run(
             [
